! =================================================================================================================================
! MODULE       : forestry
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Gathers the main elements for forest management: the "forestry"
!! subroutine, which itself calls a set of subroutines
!! forestry_clear, clearcut, thinning, harvest, force_load, QsortC, and
!! Partition, and a set of functions used in these subroutines.
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)   :
!! - Asael, S., 1999. Typologie des peuplements forestiers du massif vosgiens. 
!! C.R.P.F. Lorraine-Alsace, Nancy, 54 p.
!! - Bellassen, V., Le Maire, G., Dhote, J.F., Viovy, N., Ciais, P., 2010. 
!! Modeling forest management within a global vegetation model – Part 1: 
!! model structure aSnd general behaviour. Ecological Modelling 221, 2458–2474.
!! - Bellassen, V., Le Maire, G., Guin, O., Dhote, J.F., Viovy, N., Ciais, P., 
!! 2011a. Modeling forest management within a global vegetation model – Part 2: 
!! model validation from tree to continental scale. Ecological Modelling 222, 
!! 57–75.
!! - Bellassen, V., Viovy, N., Luyssaert, S., Le Maire, G., Schelhaas, M.J., 
!! Ciais, P., 2011b. Reconstruction and attribution of the carbon sink of 
!! European forests between 1950 and 2000. Global Change Biology 17, 3274-3292.
!! - Bottcher, H., Kurz, W.A., Freibauer, A., 2008. Accounting of forest carbon
!! sinks and sources under a future climate protocol-factoring out past 
!! disturbance and management effects on age-class structure. Environmental 
!! Science & Policy 11, 669-686.
!! - Cazin, A., Vallet, P., Dhote, J.F., 2003. Propositions LERFoB pour CARBOFOR
!! Atelier modélisation.
!! - Condes, S., Sterba, H., 2005. Derivation of compatible crown width 
!! equations for some important tree species of Spain. Forest Ecology and 
!! Management 217, 203-218.
!! - Deleuze, C., Pain, O., Dhote, J.F., Herve, J.C., 2004. A flexible radial
!! increment model for individual trees in pure even-aged stands. Annals of 
!! Forest Science 61, 327-335.
!! - Dhôte, J.-F., Hervé, J.-C., 2000. Changements de productivité dans quatre 
!! forêts de chênes sessiles depuis 1930 : une approche au niveau du peuplement. 
!! Ann. For. Sci. 57, 651-680.
!! - Dhôte, J.F., 1999. Compétition entre classes sociales chez le chêne sessile
!! et le hêtre. Revue Forestière Française, 309-325.
!! - Dhôte, J.F., Le Moguédec, G., 2003. Présentation du modèle Fagacées. 
!! INRA, 31 p.
!! - IFN, 2008. Raw inventory data - 2005-2007 inventory campaigns, www.ifn.fr.
!! - Kolari, P., Pumpanen, J., Rannik, U., Ilvesniemi, H., Hari, P., Berninger,
!! F., 2004. Carbon balance of different aged Scots pine forests in Southern 
!! Finland. Global Change Biology 10, 1106-1119.
!! - Lanier, L., 1994. Précis de sylviculture. Ecole Nationale du Génie Rural, 
!! des Eaux et des Forêts (ENGREF), Nancy, 477 p.
!! - Law, B.E., Sun, O.J., Campbell, J., Van Tuyl, S., Thornton, P.E., 2003. 
!! Changes in carbon storage and fluxes in a chronosequence of ponderosa pine. 
!! Global Change Biology 9, 510-524.
!! - Liberloo, M., Calfapietra, C., Lukac, M., Godbold, D., Luos, Z.B., Polle, 
!! A., Hoosbeek, M.R., Kull, O., Marek, M., Raines, C., Rubino, M., Taylor, G., 
!! Scarascia-Mugnozza, G., Ceulemans, R., 2006. Woody biomass production during 
!! the second rotation of a bio-energy Populus plantation increases in a future 
!! high CO2 world. Global Change Biology 12, 1094-1106.
!! - Liberloo, M., Luyssaert, S., Bellassen, V., Djomo, S.N., Lukac, M., 
!! Calfapietra, C., Janssens, I., Hoosbeek, M.R., Viovy, N., Churkina, G., 
!! Scarascia-Mugnozza, G., Ceulemans, R., 2010. Bio-energy retains its 
!! mitigation potential under elevated CO2. PLOS-one 5, e1 1648.
!! - Litton, C.M., Raich, J.W., Ryan, M.G., 2007. Carbon allocation in forest 
!! ecosystems. Global Change Biology 13, 2089-2109.
!! - Luyssaert, S., Inglima, I., Jung, M., Richardson, A.D., Reichsteins, M., 
!! Papale, D., Piao, S.L., Schulzes, E.D., Wingate, L., Matteucci, G., Aragao, 
!! L., Aubinet, M., Beers, C., Bernhoffer, C., Black, K.G., Bonal, D., 
!! Bonnefond, J.M., Chambers, J., Ciais, P., Cook, B., Davis, K.J., Dolman, 
!! A.J., Gielen, B., Goulden, M., Grace, J., Granier, A., Grelle, A., Griffis,
!! T., Grunwald, T., Guidolotti, G., Hanson, P.J., Harding, R., Hollinger, D.
!! Y., Hutyra, L.R., Kolar, P., Kruijt, B., Kutsch, W., Lagergren, F., 
!! Laurila, T., Law, B.E., Le Maire, G., Lindroth, A., Loustau, D., Malhi, Y.,
!! Mateus, J., Migliavacca, M., Misson, L., Montagnani, L., Moncrieff, J., 
!! Moors, E., Munger, J.W., Nikinmaa, E., Ollinger, S.V., Pita, G., 
!! Rebmann, C., Roupsard, O., Saigusa, N., Sanz, M.J., Seufert, G., Sierra, C., 
!! Smith, M.L., Tang, J., Valentini, R., Vesala, T., Janssens, I.A., 2007. CO2 
!! balance of boreal, temperate, and tropical forests derived from a global 
!! database. Global Change Biology 13, 2509-2537.
!! - Mokany, K., Raison, R.J., Prokushkin, A.S., 2006. Critical analysis of 
!! root: shoot ratios in terrestrial biomes. Global Change Biology 12, 84-96.
!! - Mund, M., Kummetz, E., Hein, M., Bauer, G.A., Schulze, E.D., 2002. Growth
!! and carbon stocks of a spruce forest chronosequence in central Europe. 
!! Forest Ecology and Management 171, 275-296.
!! - Newton, R.F., Amponsah, I.G., 2007. Comparative evaluation of five height-
!! diameter models developed for black spruce and jack pine stand-types in 
!! terms of goodness-of-fit, lack-of-fit and predictive ability. Forest Ecology
!! and Management 247, 149-166.
!! - Numerical Recipes, 2007. Numerical Recipes: The Art of Scientific Computing. 
!! Cambridge University Press, 1256 p.
!! - Ovington, J.D., Madgwick, H.A.I., 1957. Afforestation and soil reaction. 
!! Journal of Soil Science 8, 141-149.
!! - Pontailler, J.Y., Ceulemans, R., Guittet, J., 1999. Biomass yield of poplar 
!! after five 2-year coppice rotations. Forestry 72, 157-163.
!! - Pretzsch, H., Biber, P., Durský, J., 2002. The single tree-based stand 
!! simulator SILVA: construction, application and evaluation. Forest Ecology and 
!! Management 162, 3.
!! - Reineke, L.H., 1933. Perfecting a stand-density index for even-aged forests. 
!! Journal of Agricultural Research 46, 627-638.
!! - Turner, D.P., Ritts, W.D., Cohen, W.B., Maeirsperger, T.K., Gower, S.T., 
!! Kirschbaum, A.A., Running, S.W., Zhao, M.S., Wofsy, S.C., Dunn, A.L., Law, B.E.,
!! Campbell, J.L., Oechel, W.C., Kwon, H.J., Meyers, T.P., Small, E.E., Kurc, S.A.,
!! Gamon, J.A., 2005. Site-level evaluation of satellite-based global terrestrial 
!! gross primary production and net primary production monitoring. Global Change 
!! Biology 11, 666-684.
!! - Vacchiano, G., Motta, R., Long, J.N., Shaw, J.D., 2008. A density management 
!! diagram for Scots pine (Pinus sylvestris L.): A tool for assessing the forest's 
!! protective effect. Forest Ecology and Management 255, 2542-2554.
!! - Vieira, I.C.G., de Almeida, A.S., Davidson, E.A., Stone, T.A., de Carvalho, 
!! C.J.R., Guerrero, J.B., 2003. Classifying successional forests using Landsat 
!! spectral properties and ecological characteristics in eastern Amazonia. Remote 
!! Sensing of Environment 87, 470-481.
!! - Zianis, D., Mencuccini, M., 2004. On simplifying allometric analyses of forest 
!! biomass. Forest Ecology and Management 187, 311-332.
!! 
!! SVN      :
!! $HeadURL: $
!! $Date: $
!! $Revision: $
!! \n
!_ ================================================================================================================================

MODULE sapiens_forestry

  ! modules used:
#ifdef __NAGFOR
  USE,INTRINSIC :: IEEE_ARITHMETIC
#endif

  USE netcdf
  USE ioipsl_para
  USE constantes
  USE pft_parameters
  USE function_library,         ONLY: wood_to_volume, wood_to_height, wood_to_cn, wood_to_circ, &
                                      Nmax, wood_to_dia, Nmaxyield, Nminyield, wood_to_qmdia
  USE interpol_help,            ONLY: aggregate_p
  USE stomate_data

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC forestry,forestry_clear,read_in_fm_map, read_in_litter_map, litter_raking

  
  LOGICAL, SAVE                 :: firstcall = .TRUE.           !! first call

CONTAINS

!! ================================================================================================================================
!! FUNCTION     : calculate_rdi
!!
!>\BRIEF        Calculate the relative density index of a stand
!!
!! DESCRIPTION   : None
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : calculate_rdi
!!
!! REFERENCE(S)   : Bellassen, V., Le Maire, G., Dhote, J.F., Viovy, N., Ciais, P., 2010. 
!! Modeling forest management within a global vegetation model – Part 1: 
!! model structure and general behaviour. Ecological Modelling 221, 2458–2474.
!!
!! FLOWCHART   : None
!! \n
!_ ================================================================================================================================
  
  FUNCTION calculate_rdi(diameters_temp,circ_class_n_temp,ivm)

 !! 0. Variable and parameter declaration

    IMPLICIT NONE

    !! 0.1 Input variables
    REAL(r_std),DIMENSION(ncirc),INTENT(in)            :: circ_class_n_temp   !! The distribution of individuals
                                                                              !! for a given grid square and PFT  
    REAL(r_std),DIMENSION(ncirc),INTENT(in)            :: diameters_temp      !! The diameters of the trunks
                                                                              !! for each circ class (m)
    INTEGER(i_std),INTENT(IN)                          :: ivm                 !! The number of the PFT
                                                                              !! (unitless)

    !! 0.2 Output variables
    REAL(r_std)                                        :: calculate_rdi       !! The RDI of the stand 
                                                                              !! (unitless, always positive)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                        :: Dg                  !! The quadratic mean diameter
    REAL(r_std)                                        :: ind_loc
    INTEGER(i_std)                                     :: icir                !! Indices

!_ ================================================================================================================================

 !! 1. Calculate the relative density index for a stand, based on
 !!    the number of trees in each circumference class and the
 !!    the diameter of a model tree in each class.
    
    ind_loc=SUM(circ_class_n_temp(:))

    Dg=zero

    DO icir=1,ncirc
       Dg=Dg+circ_class_n_temp(icir)*m2_to_ha*(diameters_temp(icir)*m_to_cm)**2
    ENDDO
    Dg=SQRT(Dg/(ind_loc*m2_to_ha))
    
    calculate_rdi=(ind_loc*m2_to_ha)/Nmax(Dg,ivm)

  END FUNCTION calculate_rdi


!! ================================================================================================================================
!! SUBROUTINE    : forestry_clear
!!
!>\BRIEF        Set the flag ::firstcall to .TRUE. and as such activate section 2 of the subroutine forestry (see below).
!! 
!_ ================================================================================================================================

  SUBROUTINE forestry_clear
    firstcall = .TRUE.
  END SUBROUTINE forestry_clear


!! ================================================================================================================================
!! SUBROUTINE    : forestry
!!
!>\BRIEF        This subroutine is the core of the forest management module. It
!! has been modified so it only decides if trees are getting cut or not.
!!
!! DESCRIPTION   : This forest management module has been changed quite a bit
!! from Valentin's original version (documented in Ballassen et al (2010).  The
!! biggest reason for these changes is that it is meant to be used with the new
!! functional allocation scheme.  This scheme already includes circumference
!! classes.  In each circumference class there are circ_class_n model trees
!! which are all identical.  Therefore, we don't need to create a picture
!! of every tree on the stand since in reality we only have ncirc trees, each
!! of which are replicated.
!!
!! In addition to this, several other processes that were done here were moved
!! to other parts of the code.  For example, trees are just scheduled to die
!! here; the actually killing of the biomass and moving the wood to the harvest
!! and litter pools is done in sapiens_kill.f90.  All tree establishment after
!! clearcutting is done in stomate_prescribe.f90.  Self-thinning is a completely
!! natural process, and therefore done with environmental mortality in 
!! stomate_mark_kill.f90.

!!++++ CHECK REFERNCE ++++
!! Liberloo et al. (2010) describes how short 
!! rotation coppices are simulated and validated.\n
!!+++++++++++++++++++++++

!! The forest management flags have changed as well.
!! FM = 1 : No human intervention (ORCHIDEE default)
!!      2 : Thinnings based on the RDI, clearcuts based on tree density,
!!              annual increment, and tree diameter.  Thinnings from above and
!!              from below are determined by the sign of thstrat.
!!      3 : Coppices
!!      4 : Short rotation coppices
!!
!! Relative density index (RDI) target of human thinning at the beginning of 
!! the rotation (unitless). This is the value aimed at by human thinning at 
!! the beginning of the rotation. It is necessarily lower than 1. The actual 
!! RDI varies around this target, + or - delta_rdi, when forest_managed = 2. 
!! When forest_managed = 3, human thinning occurs every year.  For FM = 4,
!! the thinnings occur as a function of stand age.
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_kill
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE forestry (npts, biomass, age_stand, last_cut, &
             circ_class_n, circ_class_kill, forest_managed, rdi,&
             circ_class_biomass, mai, pai, previous_wood_volume, &
             mai_count, coppice_dens, veget_max, rdi_target_upper,&
             rdi_target_lower)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                             :: npts                !! Domain size - number of pixels 
                                                                                  !! (dimensionless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)           :: veget_max           !! "maximal" coverage fraction of a PFT on 
                                                                                  !! the ground (unitless, 0-1) 
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                      
                                          INTENT(in)       :: circ_class_biomass  !! Biomass components of the model tree  
                                                                                  !! within a circumference class
                                                                                  !! class @tex $(g C ind^{-1})$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(in)         :: last_cut            !! Years since last thinning (years)
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(in)         :: age_stand           !! Age of stand (years)

    !! 0.2 Output

    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)  :: circ_class_n        !! Number of trees in each circumference 
                                                                                  !! class (0-2.2+ and poles-large wood, see 
                                                                                  !! part 12.) 
       

    !! 0.3 Modified fields

    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout) :: biomass             !! Biomass @tex $(gC m^{-2})$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(inout)     :: forest_managed      !! Forest management flag: 0 = orchidee 
                                                                                  !! standard, 1= self-thinning only, 2= 
                                                                                  !! high-stand, 3= high-stand smoothed, 4= 
                                                                                  !! coppices 
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times), &
                                             INTENT(inout) :: circ_class_kill     !! Number of trees within a circ that needs
                                                                                  !! to be killed @tex $(ind m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)        :: mai                 !! The mean annual increment
                                                                                  !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)        :: pai                 !! The period annual increment
                                                                                  !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)        :: previous_wood_volume!! The volume of the tree trunks
                                                                                  !! in a stand for the previous year.
                                                                                  !! @tex $(m**3 / m**2 )$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)      :: mai_count           !! The number of times we've
                                                                                  !! calculated the volume increment
                                                                                  !! for a stand
    REAL(r_std), DIMENSION(npts,nvm),INTENT(inout)         :: coppice_dens        !! The density of a coppice at the first
                                                                                  !! cutting.
                                                                                  !! @tex $( 1 / m**2 )$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)        :: rdi                 !! Relative density index (unitless, 0-2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)        :: rdi_target_upper    !! Upper limit of RDI. When reached the stand
                                                                                  !! needs to be thinned (unitless)   
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)        :: rdi_target_lower    !! Lower limit of RDI. When thinned, thin to this             
                                                                                  !! stand density (unitless)

    !! 0.4 Local variables

    INTEGER(i_std)                                         :: icir,ivm,ipts       !! Indexes
    INTEGER(i_std)                                         :: imbc, iele, ipar    !! Indexes
    REAL(r_std)                                            :: trees,total_trees   !! a number of trees
    REAL(r_std), DIMENSION(ncirc)                          :: diameters_temp      !! Trunk diameter calculated from allometry [m]
    REAL(r_std), DIMENSION(ncirc)                          :: circ_temp           !! Trunk circumference calculated from allometry
    REAL(r_std), DIMENSION(npts,nvm,ncirc)                 :: height,dia,cn       !! Tree height calculated from allometric 
                                                                                  !! relationships (m)
    REAL(r_std)                                            :: ave_tree_height
    REAL(r_std)                                            :: woody_biomass
    REAL(r_std)                                            :: current_wood_volume
    REAL(r_std)                                            :: current_increment
    REAL(r_std)                                            :: ave_tree_dia
    INTEGER(i_std)                                         :: nrotations          !! The number of rotations that we've done
                                                                                  !! for SRC minus one
    REAL(r_std)                                            :: qm_dia              !! quadratic mean diameter of the forest (m)

!_ ================================================================================================================================

    IF ( bavard >= 2 ) WRITE(numout,*) 'Entering forestry'

 !! 1. Initialize

    ! 1.1 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 4.
    ! We don't actually have to do this in forestry, since none of the pools are
    ! changed.  The sole purpose of this routine is to schedule trees for killing
    ! which means that only circ_class_kill is changed.

    ! initialize all of the output variables...we will give them strange values
    ! instead of zero so that we know if something has gone wrong later on
!!$    rdi(:,:) = val_exp
!!$    rdi_target_upper(:,:) = val_exp
!!$    rdi_target_lower(:,:) = val_exp


    ! for every forest stand in the model, we need to decide if we are thinning or clearcutting.
    DO ipts=1,npts

       pft:DO ivm=1,nvm

          IF(.NOT. is_tree(ivm)) CYCLE

          IF(veget_max(ipts,ivm) == zero)THEN
              ! this vegetation type is not present, so no reason to do the 
              ! calculation
              CYCLE
          ENDIF

          IF(SUM(biomass(ipts,ivm,:,icarbon)) .LE. min_stomate) CYCLE
          
          ! does our management strategy allow for thinning?
          IF ( forest_managed(ipts,ivm) > 1 ) THEN

             IF(ld_forestry)THEN
                WRITE(numout,*) 'Forestry, check whether the stand needs management', ivm
             ENDIF

             ! Now we need to recompute the mean annual increment and the
             ! periodic annual increment.  For this, we need to convert the 
             ! aboveground woody biomass into a wood volume, removing the influence
             ! of the branches.
!!$          !+++CHECK+++
!!$          This calculation has been included in the library
!!$             woody_biomass=(biomass(ipts,ivm,isapabove,icarbon)+&
!!$                  biomass(ipts,ivm,iheartabove,icarbon))*&
!!$                  (un - branch_ratio(ivm))
!!$          !+++++++++++
             current_wood_volume=wood_to_volume(biomass(ipts,ivm,:,icarbon),&
                  ivm,branch_ratio(ivm),0)

             ! The mean annual increment is tricky.  I want to calculate
             ! for the volume, since that is what foresters use and the NPP
             ! would include leaf NPP.  However, we cannot calculate a
             ! volume increment immediately after thinning.  Instead, the first
             ! year after a thinning or after a clearcut (for consistency)
             ! we just save the current volume.
             IF(last_cut(ipts,ivm) .EQ. 1)THEN

                ! do nothing.  We'll save the current wood volume later.
                
             ELSEIF(last_cut(ipts,ivm) .GT. 1)THEN

                current_increment=current_wood_volume-previous_wood_volume(ipts,ivm)

                mai_count(ipts,ivm)=mai_count(ipts,ivm)+1
                mai(ipts,ivm)=mai(ipts,ivm)*REAL(mai_count(ipts,ivm)-1)/&
                     REAL(mai_count(ipts,ivm))+current_increment/REAL(mai_count(ipts,ivm))

                ! The periodic annual increment is an average over the past few
                ! years.  Notice that the first year after a cut that we can
                ! compute an increment for is year 2 (the difference between 
                ! year 2 and year 1).  
                
                IF(last_cut(ipts,ivm) .LT. (n_pai+1))THEN
                   
                   pai(ipts,ivm)=pai(ipts,ivm)*REAL(last_cut(ipts,ivm)-2)/&
                        REAL(last_cut(ipts,ivm)-1)+&
                        current_increment/REAL(last_cut(ipts,ivm)-1)

                ELSE

                   pai(ipts,ivm)=pai(ipts,ivm)*REAL(n_pai-1)/REAL(n_pai)+&
                        (current_wood_volume-previous_wood_volume(ipts,ivm))&
                        /REAL(n_pai)

                ENDIF

             ELSE

                ! I cannot think of a scenario where last_cut is zero.
                ! If it is zero, we thinned or clearcut, but that happens
                ! after this routine and we always increment it before this
                ! routine.
                WRITE(numout,*) 'Forestry, why is last_cut equal to zero?'
                WRITE(numout,*) 'ipts, ivm, last_cut(ipts,ivm) ',&
                     ipts, ivm, last_cut(ipts,ivm)
                STOP
             ENDIF

             ! We also need to store the wood volume of this year in order to 
             ! calculate the PAI for next year.
             previous_wood_volume(ipts,ivm)=current_wood_volume

             ! The following is specific for each forest management strategy
             IF(forest_managed(ipts,ivm) == ifm_thin)THEN

                ! Forests can only be thinned if it is profitable for the managers to do so.  
                ! This profitability is based on the average height of the 100 tallest trees
                ! per hectare.  If it's profitable, the stand is thinned to a target relative density
                ! index (RDI), which is the ratio of the true density to the self thinning 
                ! density.  This target depends on the management type, but it is always less
                ! than unity.  Table 1 in Valentin et al, Ecological Modeling (2010) gives values
                ! for some of these parameters.

                ! First, calculate the average height of the 100 tallest trees on the hectare.
                ! Remember that all our trees are the same height in a given circ class.
                ! Since we don't explicitly track trees, we just start at the largest circ
                ! class and add trees to our average until we have 100
                height(ipts,ivm,:)=wood_to_height(circ_class_biomass(ipts,ivm,:,:,icarbon), ivm)

                IF(ld_forestry)THEN
                   WRITE(numout,*) 'Forestry, check for thinning', ivm
                   ! just for printing out some information
                   cn(ipts,ivm,:)=wood_to_cn(circ_class_biomass(ipts,ivm,:,:,icarbon), ivm)
                   dia(ipts,ivm,:)=wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon), ivm)
                   DO icir=1,ncirc     
                      WRITE(numout,*) circ_class_n(ipts,ivm,icir)*m2_to_ha,height(ipts,ivm,icir), &
                           deux*SQRT(cn(ipts,ivm,icir)/pi), dia(ipts,ivm,icir)
                   ENDDO
                ENDIF

                total_trees=zero
                ave_tree_height=zero
                tree:DO icir=ncirc,1,-1
                   trees=circ_class_n(ipts,ivm,icir)*m2_to_ha
                   IF(trees .GE. ntrees_profit)THEN
                      ave_tree_height=height(ipts,ivm,icir)
                      EXIT tree
                   ELSEIF((trees + total_trees) .GE. ntrees_profit)THEN
                      ave_tree_height=ave_tree_height*total_trees/REAL(ntrees_profit,r_std)+&
                           (REAL(ntrees_profit,r_std)-total_trees)/REAL(ntrees_profit,r_std)*&
                           height(ipts,ivm,icir)
                      EXIT tree
                   ELSE
                      ave_tree_height=ave_tree_height*total_trees/(total_trees+trees)+&
                           trees/(total_trees+trees)*height(ipts,ivm,icir)
                      total_trees=total_trees+trees
                   ENDIF

                ENDDO tree

                ! this is our condition for thinning
                IF(ave_tree_height >= h_first(ivm))THEN

                   IF(ld_forestry)THEN
                      WRITE(numout,*) 'Forestry, check for thinning - tree height OK', ivm
                   ENDIF

                   ! in order to thin, we need to know the relative density index (RDI) of
                   ! our stand.  This is the ratio of the current stand density to
                   ! the self-thinning density, Nmax.
                   diameters_temp(:)=wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)

                   ! with these diameters, we can check to see if we want to clearcut.
                   ! If the diameters get too big, harvesting equipment can't handle the
                   ! trees anymore.  On the other hand, we need to make sure there
                   ! are enough big trees that it is worthwhile to cut them down.

                   total_trees=zero
                   DO icir=1,ncirc
                      IF(diameters_temp(icir) .GT. largest_tree_dia(ivm)) THEN
                         total_trees=total_trees+circ_class_n(ipts,ivm,icir)*m2_to_ha
                      ENDIF
                   ENDDO

                   IF(total_trees .GT. ntrees_dia_profit(ivm))THEN

                      IF(ld_forestry)THEN
                         WRITE(numout,*) 'Forestry, diameter exceeds treshold, clearcut', ivm
                         WRITE(numout,*) 'total_trees: ', total_trees
                         WRITE(numout,*) 'ntrees_dia_profit: ', ntrees_dia_profit(ivm)
                         WRITE(numout,*) 'largest_tree_dia: ', largest_tree_dia(ivm)
                         WRITE(numout,*) 'diameters_temp: ', diameters_temp(:)
                         WRITE(numout,*) 'circ_class_n (in trees per hectare): ', &
                              circ_class_n(ipts,ivm,:)*m2_to_ha
                      ENDIF
                      CALL clearcut_harvest(circ_class_n(ipts,ivm,:), &
                           circ_class_kill(ipts,ivm,:,ifm_thin,icut_clear))

                      CYCLE pft
                   ENDIF

                   ! Another condition of a clearcut is if the tees start growing 
                   ! too slowly. This is measured by looking at the mean annual 
                   ! increment, which is simply the trunk volume divided by the
                   ! age of the tree.  We compare this to average tree growth over
                   ! the past five years (i.e. the tree volume divided by 5), called
                   ! the periodic annual increment.  If the PAI is smaller than the
                   ! MAI, the trees are past their prime and the owner will cut
                   ! them and plant anew.

                   ! We calculate the MAI and the PAI at the same place where we 
                   ! increment the age of the stand, so here we just have to
                   ! compare them.  We need to check that we have enough years from
                   ! the last cut to calculate a good PAI.

                   IF(PAI(ipts,ivm) .LT. MAI(ipts,ivm) .AND. &
                        last_cut(ipts,ivm) .GT. n_pai)THEN

                      IF(ld_forestry)THEN
                         WRITE(numout,*) 'Forestry, harvesting increment PAI<MAI'
                         WRITE(numout,*) 'ipts, ivm, mai, pai: ',ipts, &
                              ivm, mai(ipts,ivm), pai(ipts,ivm)
                      ENDIF

                      CALL clearcut_harvest(circ_class_n(ipts,ivm,:), &
                           circ_class_kill(ipts,ivm,:,ifm_thin,icut_clear))
                      CYCLE pft
                   ENDIF
		   
                   ! our rdi_target will become a function of both PFT and diameter/density
                   ! Calculate rdi and rdi_target and delta_rdi based on the yield models
                   ! rdi of the stand as it is
                   qm_dia = wood_to_qmdia(circ_class_biomass(ipts,ivm,:,:,icarbon), circ_class_n(ipts,ivm,:), ivm)

                   ! The self-thinning and yield relationships are for diameters expressed in cm
                   rdi(ipts,ivm) = (SUM(circ_class_n(ipts,ivm,:))*m2_to_ha)/Nmax(qm_dia*m_to_cm,ivm)  
                   
                   ! rdi_target_upper (based on the yield tables)
                   ! rdi depends on the observed self-thinning relationship and rdi_target_upper

                   rdi_target_upper(ipts,ivm) = Nmaxyield(qm_dia*m_to_cm,ivm)/Nmax(qm_dia*m_to_cm,ivm)
                  
                   IF(ld_forestry)THEN
                      WRITE(numout,*) 'rdi', rdi(:,:)
                      WRITE(numout,*) 'rdi_target_upper calcul', rdi_target_upper(:,:)
                   ENDIF

                   ! depends on the yield tables. Because the data are not perfect and the model
                   ! fitted to the yield table data is rather limited it happens that rdi_target_upper
                   ! exceeds rdi. This is nonesensical from a biological point of view. Either
                   ! we have to use more flexible statistical models to fit to the yield data or
                   ! we need a better algorithm that takes the above constraint into account or
                   ! we simply use the minimum of the two. The last solution is here implemented by
                   ! prescribing an rdi_limit_upper. In theory this value is one but because self-
                   ! thinning is checked daily and thinning annually this condition is rarely 
                   ! satisfied. So to make sure thinning keeps happening set the limit to a value 
                   ! just below 1 i.e. 0.9

                                   
                   
                   ! rdi_target_lower (based on the yield tables). If the lower rdi for harvest is

                   rdi_target_upper(ipts,ivm) = MIN(rdi_target_upper(ipts,ivm), rdi_limit_upper)

                  ! WRITE(numout,*) 'rdi_target_upper under harvest', rdi_target_upper(:,:)
            
	           ! rdi_target_lower (based on the yield tables). If the lower rdi for harvest is
                   ! also above self-thinning there won't be any thinning because then the upper
                   ! and lower harvest rdi are equal. This is an undesired situation but we have
                   ! to deal with the numerics to avoid stupid results.

                   ! rdi_target_lower (based on the yield tables)
                   rdi_target_lower(ipts,ivm) = Nminyield(qm_dia*m_to_cm,ivm)/Nmax(qm_dia*m_to_cm,ivm)
                  
                   !WRITE(numout,*) 'rdi_target_lower calcul', rdi_target_lower(:,:)
                   
                   rdi_target_lower(ipts,ivm) = MIN(rdi_target_lower(ipts,ivm), rdi_limit_upper)
                   
          
                   !WRITE(numout,*) 'rdi_target_lower under self_thinning', rdi_target_lower(:,:)

                   !+++TEMP+++
!!$                   ! Compare old and new calculations
!!$                   IF ((rdi(ipts,ivm) - calculate_rdi(diameters_temp(:),circ_class_n(ipts,ivm,:),ivm) .GT. min_stomate) .OR. &
!!$                        (rdi(ipts,ivm) - calculate_rdi(diameters_temp(:),circ_class_n(ipts,ivm,:),ivm) .LT. -min_stomate)) THEN
!!$                      WRITE(numout,*) 'WARNING: forestry, compare rdi (new vs old), ', ivm, rdi(ipts,ivm), &
!!$                           calculate_rdi(diameters_temp(:),circ_class_n(ipts,ivm,:),ivm)
!!$                   ENDIF
!!$                   WRITE(numout,*) 'rdi_target_upper, ', ivm, rdi_target_upper
!!$                   WRITE(numout,*) 'rdi_target_lower, ', ivm, rdi_target_lower
                   !+++++++++++

                   ! we only need to thin if the rdi of our stand is outside the acceptable
                   ! rdi range given by rdi_target_upper and rdi_target_lower
                   IF(ld_forestry)THEN
                      IF(ivm == test_pft .AND. ipts == test_grid)THEN
                         WRITE(numout,*) 'Forestry, rdi print ',rdi(ipts,ivm),ipts,&
                              ivm, rdi_target_upper(ipts,ivm),&
                              rdi_target_lower(ipts,ivm)
                         WRITE(numout,*) "qm_dia,Nminyield ",qm_dia,Nminyield(qm_dia*m_to_cm,ivm)
                         WRITE(numout,*) "qm_dia,Nmaxyield ",qm_dia,Nmaxyield(qm_dia*m_to_cm,ivm)
                         WRITE(numout,*) "alpha_rdi_upper,beta_rdi_upper ",alpha_rdi_upper(ivm),beta_rdi_upper(ivm)
                         WRITE(numout,*) "nmaxtrees,dens_target ",nmaxtrees(ivm),dens_target(ivm)
                      ENDIF
                   ENDIF

                   IF((rdi(ipts,ivm) .GT. (rdi_target_upper(ipts,ivm))))THEN

                      ! we always thin down to the lower end of this range, which then gives our forest
                      ! a few years to grow before we need to thin again

                      IF(ld_forestry)THEN
                         DO icir=1,ncirc
                            WRITE(numout,*) 'Forestry, ',height(ipts,ivm,icir),circ_class_n(ipts,ivm,icir)*m2_to_ha
                         ENDDO
                      ENDIF

                      ! we need the circumference of each model tree in meters 
                      circ_temp(:)=wood_to_circ(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)

                      CALL thinning(taumin(ivm),taumax(ivm),thinstrat(ivm),diameters_temp(:),&
                           circ_temp(:),circ_class_kill(ipts,ivm,:,ifm_thin,icut_thin),circ_class_n(ipts,ivm,:),&
                           rdi_target_lower(ipts,ivm),ivm)

                      IF(ld_forestry)THEN
                         WRITE(numout,*) 'Forestry, thinning ',circ_class_kill(ipts,ivm,:,ifm_thin,icut_thin)
                         WRITE(numout,*) 'circ_class_n(ipts,ivm,:) ',circ_class_n(ipts,ivm,:)
                         DO icir=1,ncirc
                            WRITE(numout,*) 'Forestry, circ_biomass(ipts,ivm,:) ',circ_class_biomass(ipts,ivm,icir,:,icarbon)
                         ENDDO
                      ENDIF

                   ENDIF


                ENDIF

             ELSEIF(forest_managed(ipts,ivm) == ifm_cop)THEN

                ! This is for coppicing.
                ! It makes no sense to coppice an evergreen tree, apparently.
                IF(.NOT. is_deciduous(ivm))THEN

                   WRITE(numout,*) '!+++++++++++++++++++++++++++++++++++++++++++++++'
                   WRITE(numout,*) 'Forestry, Coppicing (FM = 3) should only be done on ' &
                        // 'deciduous trees.'
                   WRITE(numout,*) 'ipts,ivm,forest_managed(ipts,ivm): ',&
                       ipts,ivm,forest_managed(ipts,ivm) 
                   !STOP
                    WRITE(numout,*) ' I am not going to stop you now.  Be sure you know what you are doing!'
                    WRITE(numout,*) '!+++++++++++++++++++++++++++++++++++++++++++++++'
                   
                ENDIF

                ! Depending on the age of the stand we have different options.
                ! The current system is to do nothing until the diameter of the
                ! trees is greater than a certain value.  Once this happens, we
                ! coppice, saving the number of trees that exist in this stand.
                ! The trees then regrow until they reach this diameter again.
                ! Some trees will have died to due mortality.  At the second
                ! (and subsequent) cuts, we redistribute the below ground root mass
                ! to have the same number of trees as the first cut, killing the
                ! fine roots and harvesting aboveground biomass.

                ! Check first to see if the average diameter of the
                ! stand is greater than an input parameter.
                diameters_temp(:)=wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)
                ave_tree_dia=zero
                DO icir=1,ncirc
                   ave_tree_dia=ave_tree_dia+diameters_temp(icir)*circ_class_n(ipts,ivm,icir)
                ENDDO
                ave_tree_dia=ave_tree_dia/SUM(circ_class_n(ipts,ivm,:))

                IF(ld_forestry)THEN
                   WRITE(numout,*) 'Forestry, Do we coppice? '
                   WRITE(numout,*) 'ipts,ivm,SUM(circ_class_n(ipts,ivm,:)) ',&
                        ipts,ivm,SUM(circ_class_n(ipts,ivm,:))
                   WRITE(numout,*) 'ave_tree_dia,coppice_diameter(ivm),coppice_dens(ipts,ivm) ',&
                        ave_tree_dia,coppice_diameter(ivm),coppice_dens(ipts,ivm)
                ENDIF

                IF(ave_tree_dia .GE. coppice_diameter(ivm) .AND. coppice_dens(ipts,ivm) .GT. zero)THEN

                   IF(ld_forestry)THEN
                      WRITE(numout,*) 'Forestry, Coppicing second cut. '
                   ENDIF

                   ! We have already coppiced once, so we know our target
                   ! tree density.  We just have to mark all the trees
                   ! for coppicing.
                   circ_class_kill(ipts,ivm,:,ifm_cop,icut_cop2)=circ_class_n(ipts,ivm,:)

                ELSEIF(ave_tree_dia .GE. coppice_diameter(ivm))THEN
                   ! this is our first time coppicing

                   IF(ld_forestry)THEN
                      WRITE(numout,*) 'Forestry, Coppicing first cut. '
                      WRITE(numout,*) 'coppice_dens(ipts,ivm): ',&
                           SUM(circ_class_n(ipts,ivm,:))
                   ENDIF

                   ! save this tree density.  We will reallocate to this in future
                   ! cuts.
                   coppice_dens(ipts,ivm)=SUM(circ_class_n(ipts,ivm,:))

                   ! Mark all trees for the first coppice cut.
                   circ_class_kill(ipts,ivm,:,ifm_cop,icut_cop1)=circ_class_n(ipts,ivm,:)


                ELSE
                   
                   ! We don't need to coppice anything.

                ENDIF

             ELSEIF(forest_managed(ipts,ivm) == ifm_src)THEN

                ! This is for short rotation coppices.

                ! It makes no sense to coppice an evergreen tree, apparently.
                IF(.NOT. is_deciduous(ivm))THEN

                   WRITE(numout,*) 'Forestry, Short rotation coppicing (FM = 4) should only be done on ' &
                        // 'deciduous trees.'
                   WRITE(numout,*) 'ipts,ivm,forest_managed(ipts,ivm): ',&
                       ipts,ivm,forest_managed(ipts,ivm) 
                   STOP
                   
                ENDIF

                ! The qualifications for doing a short rotation coppice are fairly
                ! simple.  It's based on the age of the stand.  We harvest all the
                ! aboveground biomass.  After a certain number of rotations, we
                ! harvest the aboveground and kill the belowground.
                ! http://www.coppiceresources.co.uk/SRC.asp says that 30 years between
                ! total harvests.

                IF(ld_forestry)THEN
                   WRITE(numout,*) 'Forestry, Do we SRC? '
                   WRITE(numout,*) 'ipts,ivm,SUM(circ_class_n(ipts,ivm,:)) ',&
                        ipts,ivm,SUM(circ_class_n(ipts,ivm,:))
                   WRITE(numout,*) 'age_stand(ipts,ivm),src_rot_length(ivm),coppice_dens(ipts,ivm): ',&
                        age_stand(ipts,ivm),src_rot_length(ivm),coppice_dens(ipts,ivm)
                   WRITE(numout,*) 'src_nrots(ivm) ',&
                        src_nrots(ivm)
                ENDIF

                IF(MOD(age_stand(ipts,ivm),src_rot_length(ivm)) == 0)THEN

                   nrotations=age_stand(ipts,ivm)/src_rot_length(ivm)

                   IF(nrotations == src_nrots(ivm))THEN

                      ! This is the final harvest, where we will kill
                      ! all belowground biomass.
                      IF(ld_forestry)THEN
                         WRITE(numout,*) 'Forestry, SRC final harvest. '
                      ENDIF

                      circ_class_kill(ipts,ivm,:,ifm_src,icut_cop3)=&
                           circ_class_n(ipts,ivm,:)

                   ELSEIF(nrotations == 1)THEN

                      ! This is the first harvest, which will lead
                      ! to coppices.

                      IF(ld_forestry)THEN
                         WRITE(numout,*) 'Forestry, SRC first harvest. '
                      ENDIF

                      ! save this tree density.  We will reallocate to this in future
                      ! cuts.
                      coppice_dens(ipts,ivm)=SUM(circ_class_n(ipts,ivm,:))
                      
                      ! Mark all trees for the first coppice cut.
                      circ_class_kill(ipts,ivm,:,ifm_src,icut_cop1)=circ_class_n(ipts,ivm,:)


                   ELSEIF(nrotations < src_nrots(ivm))THEN

                      ! This is a standard harvest where we take the
                      ! aboveground biomass and leave the belowground.

                      IF(ld_forestry)THEN
                         WRITE(numout,*) 'Forestry, SRC standard harvest. '
                      ENDIF

                      ! Mark all trees for a standard coppice cut.
                      circ_class_kill(ipts,ivm,:,ifm_src,icut_cop2)=circ_class_n(ipts,ivm,:)

                  ELSE

                      WRITE(numout,*) 'Forestry, How did we arrive here? We must have '
                      WRITE(numout,*) 'somehow skipped the harvest in SRC. '
                      WRITE(numout,*) 'nrotations,src_nrots(ivm): ',&
                           nrotations,src_nrots(ivm)
                      STOP

                   ENDIF ! check to see how many rotations we've done


                ENDIF ! checking if it's a harvest year


             ELSE

                WRITE(numout,*) 'Forestry, We do not have any action for this management strategy'
                WRITE(numout,*) 'forest_managed(ipts,ivm),ipts,ivm ', &
                     forest_managed(ipts,ivm),ipts,ivm
                STOP 

             ENDIF ! test of management strategy


          END IF

       ENDDO pft
    ENDDO

    IF(ld_forestry)THEN
       WRITE(numout,*) 'Forestry, How many trees do we kill?'
       WRITE(numout,*) 'Name  icir  test_pft  circ_class_kill  '
       DO icir=1,ncirc
          WRITE(numout,*) 'Thin ',icir,test_pft,circ_class_kill(test_grid,test_pft,:,ifm_thin,icut_thin)
          WRITE(numout,*) 'Clearcut ',icir,test_pft,circ_class_kill(test_grid,test_pft,:,ifm_thin,icut_clear)
          WRITE(numout,*) 'Coppice 1 ',icir,test_pft,circ_class_kill(test_grid,test_pft,:,ifm_cop,icut_cop1)
          WRITE(numout,*) 'Coppice 2 ',icir,test_pft,circ_class_kill(test_grid,test_pft,:,ifm_cop,icut_cop2)
          WRITE(numout,*) 'SRC 1 ',icir,test_pft,circ_class_kill(test_grid,test_pft,:,ifm_src,icut_cop1)
          WRITE(numout,*) 'SRC 2 ',icir,test_pft,circ_class_kill(test_grid,test_pft,:,ifm_src,icut_cop2)
          WRITE(numout,*) 'SRC 3 ',icir,test_pft,circ_class_kill(test_grid,test_pft,:,ifm_src,icut_cop3)
       ENDDO
    ENDIF

 !! 4. Check mass balance closure
   
    !  Consider this whole routine as a black box with incoming and outgoing 
    !  fluxes and a change in the mass of the box. Express in absolute 
    !  units gC (or gN), hence, multiply with dt and veget_max. In most routines 
    !  veget_max does not change and could be omitted but a general approach 
    !  was prefered.

    ! We don't actually have to do this in forestry, since none of the pools are
    ! changed.  The sole purpose of this routine is to schedule trees for killing
    ! which means that only circ_class_kill is changed.

    IF (bavard >= 2) WRITE(numout,*) 'Leaving forestry'

  END SUBROUTINE forestry


!! ================================================================================================================================
!! SUBROUTINE    : clearcut_harvest
!!
!>\BRIEF        Marks all the trees at this point and this PFT for clearing
!!
!! DESCRIPTION   : This marks all the trees in this stand for killing.  The
!!                 difference between this and clearcut_noharvest is that
!!                 these trees are moved into the harvest pools in stomate_kill,
!!                 where only the branches should be left onsite.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_kill; the number of trees in each circumference class to kill
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART    : 
!! \latexonly 
!! \includegraphics[scale=0.5]{clearcutflow.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE clearcut_harvest(circ_class_n_temp, circ_class_kill_temp)

    IMPLICIT NONE

 !! 0  Variable and parameter declaration

    !! 0.1 Input variables
    REAL(r_std), DIMENSION(ncirc), INTENT(IN)            :: circ_class_n_temp          !! Number of trees in each circumference 
                                                                                       !! class 
    !! 0.2 Output variables
    REAL(r_std), DIMENSION(ncirc), INTENT(OUT)  :: circ_class_kill_temp                !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex  

    !! 0.3 Modified variables

    !! 0.4 Local variables
   

!_ ================================================================================================================================


    ! This routine has gotten much simpler.  All we want to do here is to
    ! indicate how many trees we want to kill.  The actual killing of
    ! the trees will be done in lpj_gap.  As this is a clearcut, we
    ! want to kill all of the trees.

    circ_class_kill_temp(:)=circ_class_n_temp(:)


  END SUBROUTINE clearcut_harvest

!! ================================================================================================================================
!! SUBROUTINE    : thinning
!!
!>\BRIEF        Thinning of the stand in gridpoint i for PFT j according to the
!! management type (self-thinning, anthropogenic thinning, smoothed anthropogenic
!! thinning or coppicing). Selects which trees are thinned in order to respect 
!! self-thinning rules or rdi_objective. Only trees with circumference lower than 
!! circ_lim can be thinned.
!!
!! DESCRIPTION   : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_ij0; circumference of all individual trees 
!! before thinning for 1 ha at gridpoint i and for PFT j (m), ::vol_thinned 
!! Volume of thinned trees including waste wood \f$(m^3 ha^{-1})\f$ all other readjusted 
!! stand-level variables: ::rdi, ::ind, ::av_height, ...
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART    : 
!! \latexonly 
!! \includegraphics[scale=0.5]{thinningflow.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE thinning(tmin,tmax,thstrat,diameters_temp,circ_temp,circ_class_kill_temp,circ_class_n_temp,&
       rdi_limit,ivm)
    
 !! 0  Variable and parameter declaration
    
    !! 0.1 Input variables
    REAL(r_std), INTENT(in)                :: tmin
    REAL(r_std), INTENT(in)                :: tmax
    REAL(r_std), INTENT(in)                :: thstrat
    REAL(r_std), INTENT(in)                :: rdi_limit
    REAL(r_std), DIMENSION(ncirc), INTENT(in)                :: diameters_temp
    REAL(r_std), DIMENSION(ncirc), INTENT(in)                :: circ_temp
    REAL(r_std), DIMENSION(ncirc), INTENT(in)                :: circ_class_n_temp
    INTEGER(i_std),INTENT(IN)                  :: ivm


    !! 0.2 Output variables
    REAL(r_std), DIMENSION(ncirc), INTENT(out)                :: circ_class_kill_temp

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(ncirc)                :: probability
    REAL(r_std), DIMENSION(ncirc)                :: circ_class_kill_pot
    REAL(r_std)                                  :: circmax
    REAL(r_std)                                  :: circmin
    REAL(r_std)                                  :: current_rdi
    REAL(r_std)                                  :: potential_rdi
    REAL(r_std)                                  :: dead_trees
    REAL(r_std)                                  :: previous_step
    INTEGER    :: icir,endcir,startcir,inccir,nsteps_tried


    

!_ ================================================================================================================================    


    ! We are going to assign a thinning probability to every circumference class, 
    ! based on Eq. 12 in Bellassen 2010.  He does it there for individual trees.
    ! The equation differs based on the thinning strategy used (from above or below).

    circ_class_kill_temp(:)=zero

    circmin=circ_temp(1)
    circmax=circ_temp(ncirc)

    probability(:)=zero
    
    DO icir=1,ncirc
       
       IF(thstrat .GE. zero)THEN
          ! thinning from below

          IF(ncirc == 1)THEN
             ! In this case, circmin=circmax and the denominator will be
             ! zero.  However, if we have only one circ class it doesn't really
             ! matter what the probability is set to, because all the biomass
             ! has to come from this circ class.  So we set it equal to the minimum
             ! probability.
             probability(icir)=tmin

          ELSE
             probability(icir)=tmin+(tmax-tmin)*((circmax-circ_temp(icir))/(circmax-circmin))**thstrat
          ENDIF

       ELSE
          ! thinning from above

          IF(ncirc == 1)THEN
             ! Same issue as above
             probability(icir)=tmin
          ELSE
             probability(icir)=tmin+(tmax-tmin)*((circ_temp(icir)-circmin)/(circmax-circmin))**ABS(thstrat)
          ENDIF

       ENDIF

    ENDDO

    ! now we need to kill enough trees to get down to the proper density.  If we
    ! are thinning from above, we start from the highest class, and from
    ! below, we start from the lowest.  One thing to keep in mind here is
    ! that it is possible that we have not removed enough trees after looping
    ! through all the circ classes one time, so we need to keep looping through 
    ! until our density is low enough.

    IF(thstrat .GE. zero)THEN
       startcir=1
       endcir=ncirc
       inccir=1
    ELSE
       startcir=ncirc
       endcir=1
       inccir=-1
    ENDIF

    thin_out:DO

       DO icir=startcir,endcir,inccir

          IF(ld_forestry .AND. ivm == test_pft)THEN
             WRITE(numout,*) 'Debugging thinning',ivm
             WRITE(numout,*) 'icir,startcir,endcir,inccir',icir,startcir,endcir,inccir
             WRITE(numout,*) 'rdi_limit',rdi_limit
          ENDIF

          ! check the density to see if we have gotten enough trees.  Notice we only want to
          ! include the trees that will still be living at the end of this step
          current_rdi=calculate_rdi(diameters_temp(:),circ_class_n_temp(:)-circ_class_kill_temp(:),ivm)
          IF( current_rdi .LE. rdi_limit)THEN      
             EXIT thin_out
          ENDIF

          ! if not, we need to kill some or all of the trees in this class
          ! this is the maximum number of trees we can kill in this circ class
          dead_trees=probability(icir)*(circ_class_n_temp(icir)-circ_class_kill_temp(icir))

          ! if we kill all these trees, can we get the RDI as low as we want?
          circ_class_kill_pot(:)=circ_class_kill_temp(:)
          circ_class_kill_pot(icir)=circ_class_kill_temp(icir)+dead_trees
          potential_rdi=calculate_rdi(diameters_temp(:),circ_class_n_temp(:)-circ_class_kill_pot(:),ivm)

          IF(ld_forestry .AND. ivm == test_pft)THEN
             WRITE(numout,*) 'potential_rdi,rdi_limit',potential_rdi,rdi_limit
          ENDIF

          IF(potential_rdi .LE. rdi_limit)THEN

             ! yes we can.  How do we know how many trees to kill, though?  The problem is
             ! that the RDI depends on the diameters of the remaining trees in a non-linear
             ! fashion.  Let us do a simple bisection search .  If we are here, we know that
             ! circ_class_kill_pot(icir) is too high.  So cut it in half and try again.
             previous_step=dead_trees/deux
             circ_class_kill_pot(icir)=circ_class_kill_temp(icir)+previous_step
             nsteps_tried=0
             inner_loop:DO

                potential_rdi=calculate_rdi(diameters_temp(:),circ_class_n_temp(:)-circ_class_kill_pot(:),ivm)
                nsteps_tried=nsteps_tried+1
                
                !+++++ CHECK +++++
                ! this value should be externalised
                ! Our RDI is within our tolerance of the target, so we're good.
                IF( ABS(potential_rdi - rdi_limit) .LE. 0.001_r_std)THEN
                   EXIT inner_loop
                ENDIF

                ! Is our potential RDI too high or too low?
                previous_step=previous_step/deux
                IF(potential_rdi .GT. rdi_limit)THEN

                   ! We need to kill more trees, so we add half of our previous step size
                   ! back.
                   circ_class_kill_pot(icir)=circ_class_kill_pot(icir)+previous_step
                                
                ELSE

                   ! We need to kill fewer trees, so we take half of our previous step size
                   ! away.
                   circ_class_kill_pot(icir)=circ_class_kill_pot(icir)-previous_step

                ENDIF


!!$                WRITE(6,*) 'pot ',circ_class_kill_pot(:)
                !+++++ CHECK +++++
                ! this value should be externalised
                IF(nsteps_tried .GT. 100)THEN
                   WRITE(6,*) 'pot ',circ_class_kill_pot(:)
                   WRITE(6,*) 'kill ',circ_class_kill_temp(:)
                   WRITE(6,*) 'n ',circ_class_n_temp(:)
                   STOP 'Trying too many steps in the bisection search in forestry thinning.'
                ENDIF
             ENDDO inner_loop

             ! Schedule the trees for killing.

             circ_class_kill_temp(icir)=circ_class_kill_pot(icir)

             EXIT thin_out

          ELSE

             ! no we can't, so we have to kill all possible trees and move onto the next class
             circ_class_kill_temp(icir)=circ_class_kill_temp(icir)+dead_trees

          ENDIF

        
       ENDDO 
          
       ! check to see if we have any trees left.  We should!  If not, there is a problem,
       ! and we need to catch it to avoid an infinite loop.
       IF( (SUM(circ_class_n_temp(:)) - SUM(circ_class_kill_temp(:))) .LE. min_stomate)THEN
          WRITE(6,*) 'kill ',circ_class_kill_temp(:)
          WRITE(6,*) 'n ',circ_class_n_temp(:)
          STOP 'We are thinning, but we have no trees left!'

       ENDIF

    ENDDO thin_out
    

  END SUBROUTINE thinning


!! ================================================================================================================================
!! SUBROUTINE    : read_in_fm_map
!!
!>\BRIEF        Read in a map that gives the forest management strategy
!!              for each pixel and each PFT.
!!
!! DESCRIPTION   :  
!!     
!!
!! FM = 1 : No human intervention (ORCHIDEE default)
!!      2 : Thinnings based on the RDI, clearcuts based on tree density,
!!              annual increment, and tree diameter.  Thinnings from above and
!!              from below are determined by the sign of thstrat.
!!      3 : Coppices
!!      4 : Short rotation coppices
!!
!!       NOTE: This routine was mostly copied from slowproc where the PFTmap is read in.
!!             Grid interpolation is used, but only to look at the nearby pixels to see
!!             see which management strategy is dominant.
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): ::forest_managed
!!
!! REFERENCE(S)   : 
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE read_in_fm_map ( npts, lalo, neighbours, resolution, contfrac, forest_managed )

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                             :: npts            !! Domain size - number of pixels 
                                                                              !! (dimensionless)
    REAL(r_std), DIMENSION(npts,2), INTENT(in)             :: lalo            !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), DIMENSION(npts,8), INTENT(in)          :: neighbours     !! 
    REAL(r_std), DIMENSION(npts,2), INTENT(in)             :: resolution      !! The size in m of each grid-box in X and Y
    REAL(r_std), DIMENSION(npts), INTENT(in)               :: contfrac        !! Fraction of continent in the grid

    !! 0.2 Output
    INTEGER(i_std), DIMENSION(npts,nvmap), INTENT(out)       :: forest_managed  !! Forest management flag: 0 = orchidee 
                                                                              !! standard, 1= self-thinning only, 2= 
                                                                              !! high-stand, 3= high-stand smoothed, 4= 
                                                                              !! coppices 

    !! 0.3 Modified fields


    !! 0.4 Local variables
    CHARACTER(LEN=80)                                      :: filename        !! A string to hold the file name
    LOGICAL                                                :: debug=.FALSE.   !! A flag to print out debugging information.
    INTEGER(i_std)                                         :: fid             !! The ID of the NetCDF file.
    INTEGER(i_std)                                         :: nb_coord        !! The number of coordinates in the NetCDF file    
    INTEGER(i_std)                                         :: nb_gat          !!
    INTEGER(i_std)                                         :: nb_var          !! The number of variables in the NetCDF file
    INTEGER(i_std)                                         :: nb_dim          !! The number of dimensions in the NetCDF file
    INTEGER(i_std)                                         :: iml, jml, lml,ivm !! indices
    INTEGER(i_std)                                         :: ip, inbv, jp    !! indices
    LOGICAL                                                :: l_ex            !! A flag which indicates if a variable
                                                                              !! exists in the NetCDF file
    REAL(r_std), ALLOCATABLE, DIMENSION(:)                 :: lat_lu, lon_lu  !! The latitude and longitude read in from
                                                                              !! the NetCDF file    
    INTEGER,DIMENSION(flio_max_var_dims)                   :: l_d_w           !! List of the dimension lengths of the variable
                                                                              !! in the NetCDF file
    INTEGER(i_std)                                         :: ipts            !! index
    INTEGER(i_std)                                         :: ALLOC_ERR       !! A flag tripped if we have an error in allocation
    REAL(r_std),DIMENSION(:,:,:),ALLOCATABLE               :: fmmap_r         !! The map read in from the NetCDF file
    INTEGER(i_std),DIMENSION(:,:,:),ALLOCATABLE            :: fmmap_i         !! The integer form of the map read in
    INTEGER(i_std)                                         :: closest_lat     !! The index of the closest latitude we found.
    INTEGER(i_std)                                         :: closest_lon     !! The index of the closest longitude we found.
    REAL(r_std)                                            :: distance        !! The distance from the current point to the 
                                                                              !! point on the map
    REAL(r_std)                                            :: closest_dist    !! The distance to the closet point we've found.
    INTEGER                                                :: large_int       !! A number which indicates that the grid data
                                                                              !! is not available
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lat_ful, lon_ful
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: mask
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: sub_area
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:)  :: sub_index
    CHARACTER(LEN=30) :: callsign
    INTEGER(i_std) :: ibvm, nbvmax,ifm, ivma
    LOGICAL ::           ok_interpol ! optionnal return of aggregate_2d
    REAL(r_std)                                            :: sum_fm,sumf
    REAL(r_std),DIMENSION(nfm_types)                       :: fm_sum
    LOGICAL ::           ltemp ! temporary logical variable
  !_ ================================================================================================================================

    IF ( bavard >= 2 ) WRITE(numout,*) 'Entering read_in_fm_map'

    ! This integer has to be large enough that it never shows up on the map, without being
    ! so large that is causes overflows.  Since none of the points on the map should be
    ! larger than the number of FM strategies we have (nfm_types), this is sufficiently big.
    large_int = nvmap*nfm_types+1

   !
    !Config Key   = FM_FILE
    !Config Desc  = Name of file from which the forest management map is to be read
    !Config If    = OK_STOMATE
    !Config Def   = FMmap.nc
    !Config Help  = The name of the file to be opened to read a forest management
    !Config         map (including a layer for every PFT) is given here. 
    !Config Units = [FILE]
    !
    filename = 'FMmap.nc'
    CALL getin_p('FM_FILE',filename)

    IF (is_root_prc) THEN
       IF (debug) THEN
          WRITE(numout,*) "Entering read_in_fm_map. Debug mode."
          WRITE (*,'(/," --> fliodmpf")')
          CALL fliodmpf (TRIM(filename))
          WRITE (*,'(/," --> flioopfd")')
       ENDIF
       CALL flioopfd (TRIM(filename),fid,nb_dim=nb_coord,nb_var=nb_var,nb_gat=nb_gat)
       IF (debug) THEN
          WRITE (*,'(" Number of coordinate        in the file : ",I2)') nb_coord
          WRITE (*,'(" Number of variables         in the file : ",I2)') nb_var
          WRITE (*,'(" Number of global attributes in the file : ",I2)') nb_gat
       ENDIF
    ENDIF
    CALL bcast(nb_coord)
    CALL bcast(nb_var)
    CALL bcast(nb_gat)

    ! This finds the number of longitude points in the file.
    IF (is_root_prc) &
         CALL flioinqv (fid,v_n="lon",l_ex=l_ex,nb_dims=nb_dim,len_dims=l_d_w) 
    CALL bcast(l_d_w)
    iml=l_d_w(1)
    WRITE(numout,*) "FM Map: iml =",iml

    ! This finds the number of latitude points in the file.
    IF (is_root_prc) &
         CALL flioinqv (fid,v_n="lat",l_ex=l_ex,nb_dims=nb_dim,len_dims=l_d_w) 
    CALL bcast(l_d_w)
    jml=l_d_w(1)
    WRITE(numout,*) "FM Map: jml =",jml

    ! Now find the number of PFTs in the file.  If this is not equal to the number
    ! of PFTs that we actually have, that's a problem and we'll crash.
    IF (is_root_prc) &
         CALL flioinqv (fid,v_n="FM_STRAT",l_ex=l_ex,nb_dims=nb_dim,len_dims=l_d_w) 
    CALL bcast(l_d_w)
    lml=l_d_w(3)

    IF (lml /= nvmap) THEN
       WRITE(numout,*) 'lml = ',lml
       WRITE(numout,*) 'nvmap = ',nvmap
       WRITE(numout,*) 'Stopping. '
       CALL ipslerr_p (3,'sapians_forestry', &
            &          'Problem with forest management strateg map.','lml /= nvmap', &
            &          '(number of pft must be equal)')
    ENDIF
    !

    ! Allocate the map that will be read in
    WRITE(numout,*) 'Reading the Forest Management strategy file'
    !
    ALLOC_ERR=-1
    ALLOCATE(fmmap_r(iml,jml,nvmap), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of fmmap_r : ",ALLOC_ERR
      STOP 
    ENDIF
    ALLOC_ERR=-1
    ALLOCATE(fmmap_i(iml,jml,nvmap), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of fmmap_i : ",ALLOC_ERR
      STOP 
    ENDIF

    ! This reads in the map that is in the file
    IF (is_root_prc) THEN
       fmmap_r(:,:,:)=large_int*2.0
       CALL fliogetv (fid,"FM_STRAT", fmmap_r, start=(/ 1, 1, 1 /), count=(/ iml, jml, nvmap /))
       ! Right now the values are all real, but they should be integers.
       ! Careful, NINT might not work if the precision of fmmap_r is not single.
       ! In that case, IDNINT should be used.
       DO ip=1,iml
          DO jp=1,jml
             DO ivma=1,nvmap
                !  There will be some fill values in here.  If we pass
                ! a large fill value to NINT, it crashes.  So let's 
                ! test for it
#ifdef __NAGFOR
                ltemp=IEEE_IS_NAN(fmmap_r(ip,jp,ivma))    
#else
                ltemp=isnan(fmmap_r(ip,jp,ivma))    
#endif
                IF(ltemp)THEN
                   fmmap_i(ip,jp,ivma)=large_int

                ELSE
                   IF(fmmap_r(ip,jp,ivma) .GE. 0.0 .AND. fmmap_r(ip,jp,ivma) < large_int)THEN
                      fmmap_i(ip,jp,ivma)=NINT(fmmap_r(ip,jp,ivma))
                   ELSE
                      ! This value should be big enough that we don't fool ourselves
                      ! below.
                      fmmap_i(ip,jp,ivma)=large_int
                   ENDIF
                ENDIF
             ENDDO
          ENDDO
       ENDDO
    ENDIF

    CALL bcast(fmmap_i)

    ! Now I need to get the latitude and longitude
    ! First, get the axes from the map file.
    ALLOC_ERR=-1
    ALLOCATE(lat_lu(jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lat_lu : ",ALLOC_ERR
      STOP 
    ENDIF
    ALLOC_ERR=-1
    ALLOCATE(lon_lu(iml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lon_lu : ",ALLOC_ERR
      STOP 
    ENDIF
    IF (is_root_prc) THEN
       CALL fliogstc (fid, x_axis=lon_lu,y_axis=lat_lu)
    ENDIF
    CALL bcast(lon_lu)
    CALL bcast(lat_lu)

    ! Now I can interpolate.  Remember that we are dealing with integer
    ! values and management strategies.  Therefore, we cannot do a strict
    ! interpolation, since that might gives values of the strategy to be
    ! 2.3, or something ridiculous.  We cannot just round the number, either,
    ! since we could have some squares with 1 (no management) and some with
    ! 3 (coppices);  An average of that would be 2 (high stands), which doesn't
    ! make any sense.  So we look to see what the most prevalent type of
    ! management of nearby pixels is, and then just take that.
    
    ALLOC_ERR=-1
    ALLOCATE(lat_ful(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lat_ful : ",ALLOC_ERR
      STOP 
    ENDIF
    ALLOC_ERR=-1
    ALLOCATE(lon_ful(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lon_ful : ",ALLOC_ERR
      STOP 
    ENDIF
    !
    DO ip=1,iml
       lon_ful(ip,:)=lon_lu(ip)
    ENDDO
    DO jp=1,jml
       lat_ful(:,jp)=lat_lu(jp)
    ENDDO
    !
    !
    WRITE(numout,*) 'Reading the FOREST MANAGEMENT file'
    !
    !

    !
    ! Mask of permitted variables.
    !
    ALLOC_ERR=-1
    ALLOCATE(mask(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of mask : ",ALLOC_ERR
      STOP 
    ENDIF
    !
    mask(:,:) = 0
    DO ip=1,iml
       DO jp=1,jml
          ! If at least one PFT has a management strategy, we should interpolate
          ! this grid point.
          sum_fm=SUM(fmmap_i(ip,jp,:))
          IF ( sum_fm .GT. min_sechiba .AND. sum_fm .LT. large_int) THEN
             mask(ip,jp) = 1
             IF (debug) THEN
                WRITE(numout,*) "update : SUM(fmmap(",ip,jp,")) = ",sum_fm
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !
    !
    ! The number of maximum vegetation map points in the GCM grid should
    ! also be computed and not imposed here.
    !
    nbvmax = 200
    !
    callsign="Forest Management map"
    !
    ok_interpol = .FALSE.
    DO WHILE ( .NOT. ok_interpol )
       WRITE(numout,*) "Projection arrays for ",callsign," : "
       WRITE(numout,*) "nbvmax = ",nbvmax
       !
       ALLOC_ERR=-1
       ALLOCATE(sub_index(npts, nbvmax,2), STAT=ALLOC_ERR)
       IF (ALLOC_ERR/=0) THEN
          WRITE(numout,*) "ERROR IN ALLOCATION of sub_index : ",ALLOC_ERR
          STOP 
       ENDIF
       sub_index(:,:,:)=0

       ALLOC_ERR=-1
       ALLOCATE(sub_area(npts, nbvmax), STAT=ALLOC_ERR)
       IF (ALLOC_ERR/=0) THEN
          WRITE(numout,*) "ERROR IN ALLOCATION of sub_area : ",ALLOC_ERR
          STOP 
       ENDIF
       sub_area(:,:)=zero
       !
       CALL aggregate_p(npts, lalo, neighbours, resolution, contfrac, &
            &                iml, jml, lon_ful, lat_ful, mask, callsign, &
            &                nbvmax, sub_index, sub_area, ok_interpol)
       !
       IF ( .NOT. ok_interpol ) THEN
          DEALLOCATE(sub_area)
          DEALLOCATE(sub_index)
       ENDIF
       !
       nbvmax = nbvmax * 2
    ENDDO
    !
    DO ipts = 1, npts
       ! For this point, we need to see what dominant FM types are
       ! nearby.
       sumf=zero
       ! We need to do this for every PFT
       DO ivma=1,nvmap
          ! If it's not a forest, we don't care.
          IF(.NOT. is_tree(start_index(ivma)))THEN
             forest_managed(ipts,ivma)=0
          ELSE
             fm_sum(:)=0.0
             DO ibvm=1, nbvmax
                ! Leave the do loop if all sub areas are treated, sub_area <= 0
                IF ( sub_area(ipts,ibvm) <= zero ) EXIT
                ip = sub_index(ipts,ibvm,1)
                jp = sub_index(ipts,ibvm,2)
                ifm=fmmap_i(ip,jp,ivma)
                fm_sum(ifm)=fm_sum(ifm)+sub_area(ipts,ibvm)
             ENDDO
             ! Whichever FM type has the most area, we use that one.
             forest_managed(ipts,ivma)=MAXLOC(fm_sum(:),1)
          ENDIF
       ENDDO
    ENDDO
    !
    WRITE(numout,*) 'forestry_update : Interpolation Done for FM map'
    !
    DEALLOCATE(fmmap_i)
    DEALLOCATE(fmmap_r)
    DEALLOCATE(lat_lu,lon_lu)
    DEALLOCATE(lat_ful,lon_ful)
    DEALLOCATE(mask)
    IF(ALLOCATED(sub_index)) DEALLOCATE(sub_index)
    IF(ALLOCATED(sub_area)) DEALLOCATE(sub_area)


   IF ( bavard >= 2 ) WRITE(numout,*) 'Leaving read_in_fm_map'

  END SUBROUTINE read_in_fm_map

!! ================================================================================================================================
!! SUBROUTINE    : read_in_litter_map
!!
!>\BRIEF        Read in a map that gives the litter demand for each pixel.
!!
!! DESCRIPTION   :  Reads in a map for the litter demand of each pixel and interpolates
!!                  to find the demand for all pixels that we are interested in.  The map
!!                  in this case has units of gC/year.  We convert it to be gC/m**2/year, since
!!                  all the other units in the model are given per square meter.  The litter
!!                  demand maps can be calculated based on, for example, the demand needed
!!                  per head of livestock for storing them indoors in the winter.  This practice
!!                  reached its peak during the 19th century and rapidly declined afterwards
!!                  in Europe.  If you are not interested in historical simulations or improved 
!!                  estimation of soil carbon pools during a historical spinup, you probably 
!!                  don't need this.
!!     
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): ::litter_demand
!!
!! REFERENCE(S)   : 
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE read_in_litter_map ( npts, lalo, neighbours, resolution, contfrac, litter_demand )

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                             :: npts            !! Domain size - number of pixels 
                                                                              !! (dimensionless)
    REAL(r_std), DIMENSION(npts,2), INTENT(in)             :: lalo            !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), DIMENSION(npts,8), INTENT(in)          :: neighbours      !! 
    REAL(r_std), DIMENSION(npts,2), INTENT(in)             :: resolution      !! The size in m of each grid-box in X and Y
    REAL(r_std), DIMENSION(npts), INTENT(in)               :: contfrac        !! Fraction of continent in the grid

    !! 0.2 Output
    REAL(r_std), DIMENSION(npts), INTENT(out)              :: litter_demand   !! The litter removed from each pixel due
                                                                              !! to litter raking at the end of the year.
                                                                              !! @tex $(gC year^{-1})$ @endtex

    !! 0.3 Modified fields


    !! 0.4 Local variables
    !
    ! PARAMETERS...taken from grid.f90.  I don't know why this variable is not in constants.f90.
    ! default resolution (m)
    REAL(r_std), PARAMETER :: default_resolution = 250000.

    CHARACTER(LEN=80)                                      :: filename        !! A string to hold the file name
    LOGICAL                                                :: debug=.TRUE.   !! A flag to print out debugging information.
    INTEGER(i_std)                                         :: fid             !! The ID of the NetCDF file.
    INTEGER(i_std)                                         :: nb_coord        !! The number of coordinates in the NetCDF file    
    INTEGER(i_std)                                         :: nb_gat          !!
    INTEGER(i_std)                                         :: nb_var          !! The number of variables in the NetCDF file
    INTEGER(i_std)                                         :: nb_dim          !! The number of dimensions in the NetCDF file
    INTEGER(i_std)                                         :: iml, jml, lml,ivm !! indices
    INTEGER(i_std)                                         :: ip, inbv, jp    !! indices
    LOGICAL                                                :: l_ex            !! A flag which indicates if a variable
                                                                              !! exists in the NetCDF file
    REAL(r_std), ALLOCATABLE, DIMENSION(:)                 :: lat_lu, lon_lu  !! The latitude and longitude read in from
                                                                              !! the NetCDF file    
    INTEGER,DIMENSION(flio_max_var_dims)                   :: l_d_w           !! List of the dimension lengths of the variable
                                                                              !! in the NetCDF file
    INTEGER(i_std)                                         :: ipts            !! index
    INTEGER(i_std)                                         :: ALLOC_ERR       !! A flag tripped if we have an error in allocation
    REAL(r_std),DIMENSION(:,:),ALLOCATABLE                 :: littermap       !! The map read in from the NetCDF file
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lat_ful, lon_ful
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: mask
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: sub_area
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:)  :: sub_index
    CHARACTER(LEN=30) :: callsign
    INTEGER(i_std) :: ibvm, nbvmax,ifm
    LOGICAL ::           ok_interpol ! optionnal return of aggregate_2d
    REAL(r_std)                                            :: area_sum, coslat
    REAL(r_std),DIMENSION(nfm_types)                       :: fm_sum
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:)    :: temp_resolution
    REAL(r_std)                :: fillvalue
    INTEGER :: i_rc,i_v,no_fill
    LOGICAL ::           ltemp ! temporary logical variable
!_ ================================================================================================================================

    IF ( bavard >= 2 ) WRITE(numout,*) 'Entering read_in_litter_map'

    litter_demand(:)=zero

   !
    !Config Key   = FM_FILE
    !Config Desc  = Name of file from which the forest management map is to be read
    !Config If    = OK_STOMATE
    !Config Def   = FMmap.nc
    !Config Help  = The name of the file to be opened to read a forest management
    !Config         map (including a layer for every PFT) is given here. 
    !Config Units = [FILE]
    !
    filename = 'litter_map.nc'
    CALL getin_p('LITTER_FILE',filename)

    IF (is_root_prc) THEN
       IF (debug) THEN
          WRITE(numout,*) "Entering slowproc_update. Debug mode."
          WRITE (*,'(/," --> fliodmpf")')
          CALL fliodmpf (TRIM(filename))
          WRITE (*,'(/," --> flioopfd")')
       ENDIF
       CALL flioopfd (TRIM(filename),fid,nb_dim=nb_coord,nb_var=nb_var,nb_gat=nb_gat)
       IF (debug) THEN
          WRITE (*,'(" Number of coordinate        in the file : ",I2)') nb_coord
          WRITE (*,'(" Number of variables         in the file : ",I2)') nb_var
          WRITE (*,'(" Number of global attributes in the file : ",I2)') nb_gat
       ENDIF
    ENDIF
    CALL bcast(nb_coord)
    CALL bcast(nb_var)
    CALL bcast(nb_gat)

    ! I need the fill value of the map.  This doesn't seem to available through
    ! the IOISPL routines that I can see.
    ! NF90_INQ_VAR_FILL seems to be only valid for NetCDF-'?  So I am going to make
    ! a HUGE assumption here about the fill value of the maps.  I hope that none of the values
    ! in the maps are above this value.
    fillvalue=1e20
    WRITE(numout,*) "WARNING: You are using a litter demand map.  I cannot determine"
    WRITE(numout,*) "         the fill value of this map.  I am assuming that the fill"
    WRITE(numout,*) "         value is 1e20 and that all your real values are below that."
    WRITE(numout,*) "         CONFIRM THIS WITH YOUR LITTER MAP!"
    

    ! This finds the number of longitude points in the file.
    IF (is_root_prc) &
         CALL flioinqv (fid,v_n="lon",l_ex=l_ex,nb_dims=nb_dim,len_dims=l_d_w) 
    CALL bcast(l_d_w)
    iml=l_d_w(1)
    WRITE(numout,*) "Litter map: iml =",iml

    ! This finds the number of latitude points in the file.
    IF (is_root_prc) &
         CALL flioinqv (fid,v_n="lat",l_ex=l_ex,nb_dims=nb_dim,len_dims=l_d_w) 
    CALL bcast(l_d_w)
    jml=l_d_w(1)
    WRITE(numout,*) "Litter map: jml =",jml

    ! Now find the number of PFTs in the file.  If this is not equal to the number
    ! of PFTs that we actually have, that's a problem and we'll crash.
    IF (is_root_prc) &
         CALL flioinqv (fid,v_n="LITTER_DEMAND",l_ex=l_ex,nb_dims=nb_dim,len_dims=l_d_w) 
    CALL bcast(l_d_w)

    !

    ! Allocate the map that will be read in
    WRITE(numout,*) 'Reading the litter file',l_d_w
    !
    ALLOC_ERR=-1
    ALLOCATE(littermap(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of littermap : ",ALLOC_ERR
      STOP 
    ENDIF
    ALLOC_ERR=-1

    ! This reads in the map that is in the file
    IF (is_root_prc) THEN
       CALL fliogetv (fid,"LITTER_DEMAND", littermap, start=(/ 1, 1 /), count=(/ iml, jml /))
    ENDIF

    CALL bcast(littermap)

    ! Now I need to get the latitude and longitude
    ! First, get the axes from the map file.
    ALLOC_ERR=-1
    ALLOCATE(lat_lu(jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lat_lu : ",ALLOC_ERR
      STOP 
    ENDIF
    ALLOC_ERR=-1
    ALLOCATE(lon_lu(iml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lon_lu : ",ALLOC_ERR
      STOP 
    ENDIF
    IF (is_root_prc) THEN
       CALL fliogstc (fid, x_axis=lon_lu,y_axis=lat_lu)
    ENDIF
    CALL bcast(lon_lu)
    CALL bcast(lat_lu)

    ! Now I can interpolate.  This should be straightforward, since
    ! we are interpolating real numbers.
    
    ALLOC_ERR=-1
    ALLOCATE(lat_ful(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lat_ful : ",ALLOC_ERR
      STOP 
    ENDIF
    ALLOC_ERR=-1
    ALLOCATE(lon_ful(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lon_ful : ",ALLOC_ERR
      STOP 
    ENDIF
    !
    DO ip=1,iml
       lon_ful(ip,:)=lon_lu(ip)
    ENDDO
    DO jp=1,jml
       lat_ful(:,jp)=lat_lu(jp)
    ENDDO
    
    ! The map is in absolute numbers.  We cannot interpolate absolute numbers, we want
    ! to add them together (or subtract them, if we are going to finer resolution).  Therefore,
    ! let's convert all the map values to per m**2 first, interpolate that, and then
    ! convert back to absolute numbers.
    ! This is ugly.  Unfortunately, we had to steal this from grid.f90, since the resolution is
    ! not passed for every grid square, just those that we are interested in.
    ALLOC_ERR=-1
    ALLOCATE(temp_resolution(iml,jml,2), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of temp_resolution : ",ALLOC_ERR
      STOP 
    ENDIF
    DO ip=1,iml
       DO jp=1,jml
          !
          ! 2 resolution
          !
          
          !
          ! 2.1 longitude
          !
          
          ! prevent infinite resolution at the pole
          coslat = MAX( COS( lat_ful(ip,jp) * pi/180. ), mincos )     
          IF ( iml .GT. 1 ) THEN
             
             IF ( ip .EQ. 1 ) THEN
                temp_resolution(ip,jp,1) = &
                     ABS( lon_ful(ip+1,jp) - lon_ful(ip,jp) ) * &
                     pi/180. * R_Earth * coslat
             ELSEIF ( ip .EQ. iml ) THEN
                temp_resolution(ip,jp,1) = &
                     ABS( lon_ful(ip,jp) - lon_ful(ip-1,jp) ) * &
                     pi/180. * R_Earth * coslat
             ELSE
                temp_resolution(ip,jp,1) = &
                     ABS( lon_ful(ip+1,jp) - lon_ful(ip-1,jp) )/2. *&
                     pi/180. * R_Earth * coslat
             ENDIF
             
          ELSE
             
             temp_resolution(ip,jp,1) = default_resolution
             
          ENDIF

          !
          ! 2.2 latitude
          !
          
          IF ( jml .GT. 1 ) THEN

             IF ( jp .EQ. 1 ) THEN
                temp_resolution(ip,jp,2) = &
                     ABS( lat_ful(ip,jp) - lat_ful(ip,jp+1) ) * &
                     pi/180. * R_Earth
             ELSEIF ( jp .EQ. jml ) THEN
                temp_resolution(ip,jp,2) = &
                     ABS( lat_ful(ip,jp-1) - lat_ful(ip,jp) ) * &
                     pi/180. * R_Earth
             ELSE
                temp_resolution(ip,jp,2) = &
                     ABS( lat_ful(ip,jp-1) - lat_ful(ip,jp+1) )/2. *&
                     pi/180. * R_Earth
             ENDIF
             
          ELSE
             
             temp_resolution(ip,jp,2) = default_resolution

          ENDIF
             
#ifdef __NAGFOR
                ltemp=IEEE_IS_NAN(littermap(ip,jp))
#else
                ltemp=isnan(littermap(ip,jp))
#endif
          IF( .NOT. ltemp)THEN

             IF(littermap(ip,jp) .LT. fillvalue)THEN
                ! Convert to per square meter for the interpolation if this is a real map value.
                littermap(ip,jp)=littermap(ip,jp)/temp_resolution(ip,jp,1)/temp_resolution(ip,jp,2)
             ENDIF
             
          ENDIF
          
       ENDDO

    ENDDO

    !
    WRITE(numout,*) 'Reading the LITTER file'
    !
    !

    !
    ! Mask of permitted variables.
    !
    ALLOC_ERR=-1
    ALLOCATE(mask(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of mask : ",ALLOC_ERR
      STOP 
    ENDIF
    !
    mask(:,:) = 0
    DO ip=1,iml
       DO jp=1,jml
          ! If the grid has any litter, we should interpolate
          ! this grid point.
#ifdef __NAGFOR
                ltemp=IEEE_IS_NAN(littermap(ip,jp))
#else
                ltemp=isnan(littermap(ip,jp))
#endif
          IF( .NOT. ltemp)THEN
             IF ( littermap(ip,jp) .GT. min_sechiba .AND. littermap(ip,jp) .LT. fillvalue ) THEN
                mask(ip,jp) = 1
                IF (debug) THEN
                   WRITE(numout,*) "update : littermap(",ip,jp,")) = ",littermap(ip,jp)
                ENDIF
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !
    !
    ! The number of maximum vegetation map points in the GCM grid should
    ! also be computed and not imposed here.
    !
    nbvmax = 200
    !
    callsign="Litter map"
    !
    ok_interpol = .FALSE.
    DO WHILE ( .NOT. ok_interpol )
       WRITE(numout,*) "Projection arrays for ",callsign," : "
       WRITE(numout,*) "nbvmax = ",nbvmax
       !
       ALLOC_ERR=-1
       ALLOCATE(sub_index(npts, nbvmax,2), STAT=ALLOC_ERR)
       IF (ALLOC_ERR/=0) THEN
          WRITE(numout,*) "ERROR IN ALLOCATION of sub_index : ",ALLOC_ERR
          STOP 
       ENDIF
       sub_index(:,:,:)=0

       ALLOC_ERR=-1
       ALLOCATE(sub_area(npts, nbvmax), STAT=ALLOC_ERR)
       IF (ALLOC_ERR/=0) THEN
          WRITE(numout,*) "ERROR IN ALLOCATION of sub_area : ",ALLOC_ERR
          STOP 
       ENDIF
       sub_area(:,:)=zero
       !
       CALL aggregate_p(npts, lalo, neighbours, resolution, contfrac, &
            &                iml, jml, lon_ful, lat_ful, mask, callsign, &
            &                nbvmax, sub_index, sub_area, ok_interpol)
       !
       IF ( .NOT. ok_interpol ) THEN
          DEALLOCATE(sub_area)
          DEALLOCATE(sub_index)
       ENDIF
       !
       nbvmax = nbvmax * 2
    ENDDO
    !
    DO ipts = 1, npts

       litter_demand(ipts)=zero
       area_sum=zero

       DO ibvm=1, nbvmax
          ! Leave the do loop if all sub areas are treated, sub_area <= 0
          IF ( sub_area(ipts,ibvm) <= zero ) EXIT
          ip = sub_index(ipts,ibvm,1)
          jp = sub_index(ipts,ibvm,2)
          area_sum=area_sum+sub_area(ipts,ibvm)
          litter_demand(ipts)=litter_demand(ipts)+sub_area(ipts,ibvm)*littermap(ip,jp)
       ENDDO

       IF(area_sum .LT. min_stomate)THEN
          WRITE(numout,*) 'Missing data for the litter map on this land point!'
          WRITE(numout,*) 'ipts,lalo',ipts,lalo(ipts,:)
          litter_demand(ipts)=zero
       ELSE
          litter_demand(ipts)=litter_demand(ipts)/area_sum
          ! This is in gC/year/m**2 from the interpolation.  I need to convert it back
          ! to absolute numbers.
          litter_demand(ipts)=litter_demand(ipts)*resolution(ipts,1)*resolution(ipts,2)
       ENDIF

    ENDDO

    !
    WRITE(numout,*) 'forestry_update : Interpolation Done for litter map'
    !
    DEALLOCATE(littermap)
    DEALLOCATE(temp_resolution)
    DEALLOCATE(lat_lu,lon_lu)
    DEALLOCATE(lat_ful,lon_ful)
    DEALLOCATE(mask)
    IF(ALLOCATED(sub_index)) DEALLOCATE(sub_index)
    IF(ALLOCATED(sub_area)) DEALLOCATE(sub_area)


   IF ( bavard >= 2 ) WRITE(numout,*) 'Leaving read_in_litter_map'

  END SUBROUTINE read_in_litter_map

!! ================================================================================================================================
!! SUBROUTINE    : litter_raking
!!
!>\BRIEF        Read in a map that gives the litter demand for each pixel.
!!
!! DESCRIPTION   :  Reads in a map for the litter demand of each pixel and interpolates
!!                  to find the demand for all pixels that we are interested in.  The map
!!                  in this case has units of gC/year.  We convert it to be gC/m**2/year, since
!!                  all the other units in the model are given per square meter.  The litter
!!                  demand maps can be calculated based on, for example, the demand needed
!!                  per head of livestock for storing them indoors in the winter.  This practice
!!                  reached its peak during the 19th century and rapidly declined afterwards
!!                  in Europe.  If you are not interested in historical simulations or improved 
!!                  estimation of soil carbon pools during a historical spinup, you probably 
!!                  don't need this.
!!     
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): ::litter_demand
!!
!! REFERENCE(S)   : 
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE litter_raking ( npts, veget_max, resolution, litter_demand, litter, lrake_frac )

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                             :: npts            !! Domain size - number of pixels 
                                                                              !! (dimensionless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)                :: veget_max       !! "Maximal" coverage fraction of a PFT (LAI 
                                                                              !! -> infinity) on ground
    REAL(r_std), DIMENSION(npts,2), INTENT(in)             :: resolution      !! The size in m of each grid-box in X and Y
    REAL(r_std), DIMENSION(npts), INTENT(in)               :: litter_demand   !! The litter removed from each pixel due
                                                                              !! to litter raking at the end of the year.
                                                                              !! @tex $(gC year^{-1})$ @endtex

    !! 0.2 Output
    REAL(r_std), DIMENSION(:,:), INTENT(OUT)                :: lrake_frac      !! TEMP


    !! 0.3 Modified fields
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: litter                  !! Metabolic and structural litter, above 
                                                                                !! and below ground 
                                                                                !! @tex $(gC m^{-2})$ @endtex 

    !! 0.4 Local variables
    INTEGER(i_std)                                   :: ipts, ivm               !! Indices
    REAL(r_std)                                      :: total_litter,ind_litter, crop_frac, forest_frac, litter_transfer
    REAL(r_std),DIMENSION(nlitt)                     :: litter_ratio
    REAL(r_std),DIMENSION(nvm)                       :: lfrac
    INTEGER(i_std) :: ncrops
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)   :: check_intern      !! Contains the components of the internal
                                                                              !! mass balance chech for this routine
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: closure_intern    !! Check closure of internal mass balance
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: pool_start        !! Start pool of this routine 
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: pool_end          !! End pool of this routine 
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std)                                          :: temp_sum          !! Temporary storage
    INTEGER(i_std)                                   :: iele, ilitt, ilevs, imbc     !! Indices
    REAL(r_std), DIMENSION(npts,nvm)                 :: pft_area              !! The absolute area covered by this PFT
                                                                              !! @tex $(m^2)$ @endtex
    REAL(r_std), DIMENSION(npts)                 :: pixel_area              !! The absolute area covered by this PFT
                                                                              !! @tex $(m^2)$ @endtex
    REAL(r_std) :: litter_lost,litter_gained
    LOGICAL :: lall_litter
!_ ================================================================================================================================

    IF ( bavard >= 2 ) WRITE(numout,*) 'Entering litter_raking'

    DO ipts=1,npts
       pft_area(ipts,:)=resolution(ipts,1)*resolution(ipts,2)* veget_max(ipts,:)
       pixel_area(ipts)=resolution(ipts,1)*resolution(ipts,2)
    ENDDO

    !! 1.1 Initialize check for mass balance closure  
    !  Initial values of litter
    ! We cannot check the litter for every PFT individually, since
    ! we are moving litter between forest and crop PFTs.
    ! We have no problem doing this every time this routine is called
    ! because it's only called at the end of the year and this calculation
    ! is not very expensive.
    pool_start(:,:,:) = zero
    
    DO iele = 1,nelements
       
       ! Litter pool (gC m-2) *  (m2 m-2) 
       DO ilitt = 1,nlitt
          DO ilevs = 1,nlevs
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  litter(:,ilitt,:,ilevs,iele) * veget_max(:,:)
          ENDDO
       ENDDO
       
       
    ENDDO ! # nelements

    lrake_frac(:,:)=zero

    ! We need to find how much litter we can get from the forests for each grid point.
    DO ipts=1,npts
       litter_lost=zero
       litter_gained=zero
       total_litter=zero
       DO ivm=1,nvm
          IF(is_tree(ivm))THEN
             ! We will take all the above ground litter, since that is what a farmer
             ! could pick up.  Unlikely they dug into the soil to collect litter.
             ! Notice these are in absolute units, not per square meter.
             total_litter=total_litter+SUM(litter(ipts,:,ivm,iabove,icarbon))&
                  *pft_area(ipts,ivm)
          ENDIF
       ENDDO

       ! If we have no forest litter, we cannot rake anything.
       IF(total_litter .LT. min_stomate)CYCLE

       ! The forests are tricky.  The litter they produce will not necessarily be proportional
       ! to the area they take up.  We want to reduce the amount of litter in each forest in a way
       ! that doesn't leave any litter pools negative.
       lfrac(:)=zero
       DO ivm=1,nvm
          IF(is_tree(ivm))THEN
             lfrac(ivm)=SUM(litter(ipts,:,ivm,iabove,icarbon))&
                  *pft_area(ipts,ivm)/total_litter
          ENDIF
       ENDDO

       ! Now compute the amount of the litter that we need to take to satisfy our demand.
       litter_transfer=litter_demand(ipts)

       ! We don't take more litter than we have.  I'm also putting in a logical flag to prevent
       ! us from having numerical issues.
       lall_litter=.FALSE.
       IF(litter_transfer .GT. total_litter) THEN
          lall_litter=.TRUE.
          litter_transfer=total_litter
       ENDIF

       ! How many crop PFTs are present on this grid square?
       crop_frac=zero
       DO ivm=1,nvm
          IF(.NOT. natural(ivm))THEN
             crop_frac=crop_frac+veget_max(ipts,ivm)
          ENDIF
       ENDDO

       IF(crop_frac .GT. min_stomate)THEN
          DO ivm=1,nvm

             IF(veget_max(ipts,ivm) .LE. min_stomate) CYCLE

             ! If we have a forest, we remove litter.  Keep the ratio of the pools the same.
             IF(is_tree(ivm))THEN
                ind_litter=SUM(litter(ipts,:,ivm,iabove,icarbon))*&
                     pft_area(ipts,ivm)
                !  I guess there is a chance that a forest has no litter.  This
                ! means there is nothing to take away.
                IF(ind_litter .LE. min_stomate)CYCLE

                litter_ratio(:)=litter(ipts,:,ivm,iabove,icarbon)*&
                     pft_area(ipts,ivm)/ind_litter
                lrake_frac(ipts,ivm)=lfrac(ivm)*litter_transfer/ind_litter
                ! We don't want to have negative litter values
                IF(lall_litter)THEN
                   ind_litter=zero
                ELSE
                   ind_litter=ind_litter-lfrac(ivm)*litter_transfer
                ENDIF
                litter_lost=litter_lost-lfrac(ivm)*litter_transfer

                ! This is a problem.  We are taking differences of numbers which are 1e9 or so, which means
                ! we could have a negative value that is insignificant.  I put the 
                ! flag to try to prevent that.
                IF(.NOT. lall_litter .AND. ind_litter .LT. -min_stomate)THEN
                   WRITE(numout,*) "Took away too much litter!"
                   WRITE(numout,*) "ipts,ivm:",ipts,ivm
                   WRITE(numout,*) "lfrac(ivm),ind_litter,litter_transfer: ",&
                        lfrac(ivm),SUM(litter(ipts,:,ivm,iabove,icarbon))*&
                        pft_area(ipts,ivm),litter_transfer
                ENDIF
                litter(ipts,:,ivm,iabove,icarbon)=litter_ratio(:)*ind_litter&
                     /pft_area(ipts,ivm)

             ELSEIF( .NOT. natural(ivm))THEN
                ! If we have a crop, we add litter.
                ind_litter=SUM(litter(ipts,:,ivm,iabove,icarbon))*pft_area(ipts,ivm)
                ! Is it possible that we have no crop litter but we have veget_max?  Maybe.
                IF(ind_litter .LT. min_stomate)THEN
                   litter_ratio(:)=un/REAL(nlitt)
                ELSE
                   litter_ratio(:)=litter(ipts,:,ivm,iabove,icarbon)*pft_area(ipts,ivm)/ind_litter
                ENDIF
                ! This is the total litter we have for this PFT in this pixel
                ind_litter=ind_litter+veget_max(ipts,ivm)/crop_frac*litter_transfer
                litter_gained=litter_gained+veget_max(ipts,ivm)/crop_frac*litter_transfer
                litter(ipts,:,ivm,iabove,icarbon)=litter_ratio(:)*ind_litter&
                     /pft_area(ipts,ivm)

             ENDIF ! checking to see if this PFT is a crop or forest


          ENDDO ! loop over PFTs

       ELSE
          ! We have no agricultural PFTs to move the litter to, so we
          ! assume that no litter raking was done in this grid square.
       ENDIF ! checking to see if we have crops

    ENDDO ! loop over pixels
    
 !! 2. Check mass balance closure
   
    !  Consider this whole routine as a black box with incoming and outgoing 
    !  fluxes and a change in the mass of the box. Express in absolute 
    !  units gC (or gN), hence, multiply with dt and veget_max. In most routines 
    !  veget_max does not change and could be omitted but a general approach 
    !  was prefered.

    !! 2.1 Calculate pools at the end of the routine
    pool_end = zero
    DO iele = 1,nelements
       DO ilitt = 1,nlitt
          DO ilevs = 1,nlevs
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  litter(:,ilitt,:,ilevs,iele) * veget_max(:,:)
          ENDDO
       ENDDO
    ENDDO

     !! 2.2 Calculate components of the mass balance
    check_intern(:,:,iatm2land,icarbon) = zero
    check_intern(:,:,iland2atm,icarbon) = -un * zero
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
    ENDDO
 
    ! Write outcome.  We sum over all the PFTs in the gridcell here.
    DO ipts=1,npts
       IF(ABS(SUM(closure_intern(ipts,:,icarbon))) .LE. min_stomate)THEN
          IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in litter_raking'
       ELSE
          WRITE(numout,*) 'Error: mass balance is not closed in litter_raking'
          DO ivm=1,nvm
             WRITE(numout,*) '   ipts,ivm; ', ipts,ivm
             WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
             WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,ivm,icarbon), pool_start(ipts,ivm,icarbon)
          ENDDO
          IF(ld_stop)THEN
             CALL ipslerr_p (3,'litter_raking', 'Mass balance error.','','')
          ENDIF
       ENDIF
    ENDDO
    
    IF ( bavard >= 2 ) WRITE(numout,*) 'Leaving litter_raking'
    
  END SUBROUTINE litter_raking

END MODULE sapiens_forestry
